package net.tomatentum.cutin.container;

import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;
import java.util.Set;

import net.tomatentum.cutin.ReflectedMethod;
import net.tomatentum.cutin.ReflectedMethodFactory;

public class LoneMethodContainer<I extends Object> implements MethodContainer<I> {

    private Map<I, ReflectedMethod<I>> methodStore;
    private ReflectedMethodFactory<I> factory;

    public LoneMethodContainer(ReflectedMethodFactory<I> factory) {
        this.methodStore = new HashMap<>();
        this.factory = factory;
    }

    @Override
    public MethodContainer<I> addMethod(ReflectedMethod<I> method) {
        this.methodStore.put(method.identifier(), method);
        return this;
    }

    @Override
    public MethodContainer<I> addMethods(Object containingObject, Method... methods) {
        for (Method method : methods)
            this.factory.produce(method, containingObject)
                .ifPresent(this::addMethod);
        return this;
    }

    @Override
    public Set<I> identifiers() {
        return methodStore.keySet();
    }

    @Override
    public Collection<ReflectedMethod<I>> methods() {
        return this.methodStore.values();
    }

    @Override
    public Collection<ReflectedMethod<I>> findFor(I identifier) {
        return Arrays.asList(this.methodStore.get(identifier));
    }

    @Override
    public Optional<ReflectedMethod<I>> findFirstFor(I identifier) {
        return findFor(identifier).stream().findFirst();
    }

}
